
/* tcptb
 * Mark Allman (mallman@icir.org)
 *
 * Copyright (c) 2004 International Computer Science Institute
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * The names and trademarks of copyright holders may not be used in
 * advertising or publicity pertaining to the software without specific
 * prior permission. Title to copyright in this software and any
 * associated documentation will at all times remain with the copyright
 * holders.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <tcptb.h>
#include <tcptb-int.h>


void tcptb_stats_accum (cnn,pkt,stats,pkti)
struct tcptb_cnn *cnn;
struct tcptb_pkt *pkt;
struct tcptb_cnn_stats *stats;
struct tcptb_pkt_info *pkti;
{
    short ep = 0;
    struct ip *iph;
    struct tcphdr *tcph;
    unsigned short i;
    struct tcptb_tcp_opts opts;

    if ((pkt == NULL) || (stats == NULL))
	return;

    /* update times */
    stats->last_time = pkt->ts.tv_sec + (pkt->ts.tv_usec / 1000000.);

    /* find packet headers */
    if ((iph = tcptb_iphdr (pkt)) == NULL)
	return;
    if ((tcph = tcptb_tcphdr (pkt)) == NULL)
	return;

    /* determine which direction this packet is travelling */
    if ((cnn->ip [1] == iph->ip_src.s_addr) && 
	(cnn->port [1] == tcph->th_sport))
	ep = 1;

    /* count packets */
    stats->tot_pkts_sent [ep]++;

    /* SYN and option accounting */
    if (tcptb_is_syn (tcph))
    {
	stats->syns_seen [ep]++;
	if ((tcptb_find_tcp_opts (tcph,&opts) > -1) && (opts.num_opts > 0))
	{
	    /* TODO: grovel for TS option, as well */
	    for (i = 0; i < opts.num_opts; i++)
		if (*opts.opts [i] == TCPOPT_SACK_PERMITTED)
		    stats->sack_ok [ep] = TRUE;
		else if (*opts.opts [i] == TCPOPT_WINDOW)
		{
		    stats->ws_seen [ep] = TRUE;
		    tcptb_get_ws_value (opts.opts [i],&stats->ws [ep]);
		}
		else if (*opts.opts [i] == TCPOPT_MAXSEG)
		    tcptb_get_mss_value (opts.opts [i],&stats->mss [ep]);
	}
    }

    /* if the user wants us to slice and dice on the current packet, do so */
    if (pkti != NULL)
    {
	bzero (pkti,sizeof (struct tcptb_pkt_info));

	/* general things */
	pkti->now = stats->last_time;
	pkti->endpoint = ep;
	pkti->other_endpoint = (ep == 0) ? 1 : 0;
	pkti->tcph = tcph;
	pkti->iph = iph;

	/* payload length of this packet */
	pkti->payload_len = iph->ip_len - (iph->ip_hl * 4) - 
	    (tcph->th_off * 4);
	
	/* sequence number of the last octet carried in this packet */
	pkti->end_seqno = tcph->th_seq;
	pkti->end_seqno += (pkti->payload_len) ? pkti->payload_len - 1 : 0;
	pkti->end_seqno += tcptb_is_syn (tcph) ? 1 : 0;
	pkti->end_seqno += tcptb_is_fin (tcph) ? 1 : 0;

	/* parse up the options */
	if (tcptb_find_tcp_opts (tcph,&pkti->opts) == -1)
	    pkti->opts.num_opts = 0;
	/* TODO: dig out timestamps, too */
	for (i = 0; i < pkti->opts.num_opts; i++)
	    if (*pkti->opts.opts [i] == TCPOPT_SACK)
		if (tcptb_get_sack_blocks (pkti->opts.opts [i],&pkti->sack,
					   tcph->th_ack) < 0)
		    pkti->sack.num_blocks = 0;
    }
}
