
/* tcptb
 * Mark Allman (mallman@icir.org)
 *
 * Copyright (c) 2004 International Computer Science Institute
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * The names and trademarks of copyright holders may not be used in
 * advertising or publicity pertaining to the software without specific
 * prior permission. Title to copyright in this software and any
 * associated documentation will at all times remain with the copyright
 * holders.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <tcptb.h>
#include <tcptb-int.h>


void tcptb_add_pkt_to_cnn (phdr,cnn,pkt)
struct pcap_pkthdr *phdr;
struct tcptb_cnn *cnn;
char *pkt;
{
    struct tcptb_pkt *newnode, *p, *q;

    newnode = (struct tcptb_pkt *)tcptb_malloc (sizeof (struct tcptb_pkt));
    if (newnode == NULL)
    {
	tcptb_set_error ("fatal error: no memory for packet",TRUE);
	return;
    }
    bzero (newnode,sizeof (struct tcptb_pkt));
    bcopy (&phdr->ts,&newnode->ts,sizeof (struct timeval));
    newnode->len = phdr->len;
    newnode->caplen = phdr->caplen;
    newnode->pkt = pkt;
    if (cnn->pkts_head == NULL)
	cnn->pkts_head = cnn->pkts_tail = newnode;
    else
    {
	if (tcptb_tvcmp (newnode->ts,cnn->pkts_tail->ts) >= 0)
	{
	    cnn->pkts_tail->next = newnode;
	    cnn->pkts_tail = newnode;
	}
	else if (tcptb_tvcmp (newnode->ts,cnn->pkts_head->ts) <= 0)
	{
	    newnode->next = cnn->pkts_head;
	    cnn->pkts_head = newnode;
	}
	else
	{
	    p = cnn->pkts_head->next;
	    q = cnn->pkts_head;
	    while (p != NULL)
	    {
		if ((tcptb_tvcmp (newnode->ts,p->ts) <= 0) &&
		    (tcptb_tvcmp (newnode->ts,q->ts) >= 0))
		{
		    q->next = newnode;
		    newnode->next = p;
		    break;
		}
		q = p;
		p = p->next;
	    }
	    if (p == NULL)
		tcptb_set_error ("could not insert out-of-order packet",TRUE);
	}
    }
}


void tcptb_process_pkt (phdr,pkt,iph,tcph)
struct pcap_pkthdr *phdr;
char *pkt;
struct ip *iph;
struct tcphdr *tcph;
{
    struct tcptb_cnn *cnn;

    if ((cnn = tcptb_cnn_known (iph->ip_src.s_addr,iph->ip_dst.s_addr,
				tcph->th_sport,tcph->th_dport)) == NULL)
	cnn = tcptb_new_cnn (iph->ip_src.s_addr,iph->ip_dst.s_addr,
			     tcph->th_sport,tcph->th_dport);
    tcptb_add_pkt_to_cnn (phdr,cnn,pkt);
}
